//$Id: form-res.h,v 1.29 2003/01/25 16:11:52 cactus Exp $ -*- c++ -*-

/* Guikachu Copyright (C) 2001-2002 RDI Gerg <cactus@cactus.rulez.org>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#ifndef GUIKACHU_FORM_RES_H
#define GUIKACHU_FORM_RES_H

namespace Guikachu
{
    namespace Resources
    {
	class Form;
    }
}

#include "resource.h"
#include "resource-ref.h"
#include "form-editor/widget.h"
#include "form-editor/widget-ref.h"

#include <set>

namespace Guikachu
{
    namespace GUI
    {
	class FormWindow;
    }
    
    namespace Resources
    {
	class Form: public Resource
	{
	    Guikachu::GUI::FormWindow *gui;

	    std::set<Widget*> widgets;

	public:
	    Form (ResourceManager *manager, const std::string &name);
	    ~Form ();
	    
	    Type get_type () const { return RESOURCE_FORM; };
	    void apply_visitor (ResourceVisitor &visitor) { visitor.visit_resource (this); };
	    void show_ui ();

	    // Widget managment
	    const std::set<Widget*> & get_widgets () const { return widgets; };
	    Widget*                   get_widget  (const std::string &id) const;
	    
	    Widget* create_widget (Widgets::Type  type,
				   std::string    id = "",
				   bool           try_alternate_names = false);
	    void    remove_widget (Widget        *widget);

	    SigC::Signal1<void, Widget*> widget_created;
	    SigC::Signal1<void, Widget*> widget_removed;
	    SigC::Signal0<void> widget_changed;

	    // Properties
	    Property<std::string>   title;
	    Properties::ResourceRef help_id;
	    Properties::ResourceRef menu_id;

	    Property<bool>          modal;
	    Property<bool>          frame;
	    Property<bool>          savebehind;
	    
	    Properties::WidgetRef   def_button;

	    Property<int>           x;
	    Property<int>           y;
	    Property<int>           width;
	    Property<int>           height;

	    // Utility functions for widgets
	    int get_screen_width () const;
	    int get_screen_height () const;
	};
    };
};

#endif /* !GUIKACHU_FORM_RES_H */
